import argparse
import json
import os
import readline
import signal
import typer

from openai_client import get_gpt_response
from prompt_builder import PromptBuilder
from shell_actions import handle_shell_action
from think_actions import handle_think_action
from utils import typer_writer

history_commands = []

history_file = os.path.join(os.getenv("HOME"),
                            ".config", "shell_gpt",
                            "history_commands.json")
try:
    readline.read_history_file(history_file)
except FileNotFoundError:
    pass


def key_handler(event):
    global history_commands
    if event == readline.SIGINT:
        # User pressed Ctrl-C, so clear the input line
        readline.set_line_buffer('')
        print('^C')
    elif event == readline.KEY_UP:
        # User pressed Up arrow, so get previous command from history
        if history_commands:
            readline.set_history_item(
                readline.get_current_history_length() - 1, history_commands.pop())
    elif event == readline.KEY_DOWN:
        # User pressed Down arrow, so get next command from history
        if readline.get_current_history_length() > len(history_commands):
            history_commands.append(readline.get_history_item(
                readline.get_current_history_length() - 1))
    elif event == readline.KEY_LEFT:
        # User pressed Left arrow, so move cursor left
        current_pos = readline.get_begidx()
        if current_pos > 0:
            readline.set_cursor_position(current_pos - 1)
    elif event == readline.KEY_RIGHT:
        # User pressed Right arrow, so move cursor right
        current_pos = readline.get_begidx()
        end_pos = readline.get_endidx()
        if current_pos < end_pos:
            readline.set_cursor_position(current_pos + 1)


def main():
    global history_commands

    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--script_path", help="Path to the script to run", required=False)
    parser.add_argument("--script_args", nargs=argparse.REMAINDER,
                        help="Arguments to pass to the script")
    parser.add_argument("--cautious", action="store_true")
    parser.add_argument("--temperature", type=float, default=1)
    args = parser.parse_args()
    script_path = args.script_path
    script_args = args.script_args
    cautious = args.cautious
    temperature = args.temperature

    prompt_builder = PromptBuilder()
    latest_response = None

    try:
        with open(os.path.join(os.getenv("HOME"), ".config", "shell_gpt", "env_var2val.json"), "r") as fp:
            env_var2val = json.load(fp)
    except FileNotFoundError:
        env_var2val = dict()

    if script_path:
        raise NotImplementedError("Script path is not implemented yet")
    else:
        readline.parse_and_bind('')
        readline.set_pre_input_hook(key_handler)
        while True:
            inp = input(">>> ")
            if inp == "exit":
                with open(os.path.join(os.getenv("HOME"),
                                       ".config", "shell_gpt",
                                       "env_var2val.json"), "w+") as fp:
                    json.dump(env_var2val, fp)
                readline.set_history_length(1000)
                readline.write_history_file(history_file)
                break

            try:
                if inp.startswith("SHELL: "):
                    inp = inp.replace("SHELL: ", "")
                    prompt = prompt_builder.shell_prompt(inp)
                    response = get_gpt_response(
                        prompt, temperature=temperature, top_p=1, caching=False, chat=None)
                    typer_writer(response, code=True,
                                 shell=True, animate=True)
                    if response.startswith("COMMAND: "):
                        response = response.replace("COMMAND: ", "")
                        if typer.confirm("Run this command?"):
                            os.system(response)
                    else:
                        print(response)
                        print(
                            "The response is not a command. This is a bug from OpenAI.")
                elif inp.startswith("DO: "):
                    response = handle_shell_action(
                        inp, env_var2val, latest_response)
                    if not response:
                        print("Action failed. Please try again.")
                elif inp.startswith("THINK: "):
                    response = handle_think_action(
                        inp, env_var2val, temperature)
                    latest_response = response
                elif inp.startswith("CODE: "):
                    inp = inp.replace("CODE: ", "")
                    prompt = prompt_builder.code_prompt(inp)
                    response = get_gpt_response(
                        prompt, temperature=temperature, top_p=1, caching=False, chat=None)
                    latest_response = response
                    typer_writer(response, code=True,
                                 shell=False, animate=True)
                if cautious:
                    print(prompt)
            except Exception as e:
                print("Something went wrong. Please try again.")
                print(e)


if __name__ == "__main__":
    main()



import json
from hashlib import md5
from pathlib import Path
from typing import List, Dict, Callable


class Cache:
    """
    Decorator class that adds caching functionality to a function.
    """

    def __init__(self, length: int, cache_path: Path) -> None:
        """
        Initialize the Cache decorator.

        :param length: Integer, maximum number of cache files to keep.
        """
        self.length = length
        self.cache_path = cache_path
        self.cache_path.mkdir(parents=True, exist_ok=True)

    def __call__(self, func: Callable) -> Callable:
        """
        The Cache decorator.

        :param func: The function to cache.
        :return: Wrapped function with caching.
        """
        def wrapper(*args, **kwargs):
            if not kwargs.pop("caching", True):
                return func(*args, **kwargs)
            # Exclude self instance from hashing.
            cache_key = md5(json.dumps(
                (args[1:], kwargs)).encode('utf-8')).hexdigest()
            cache_file = self.cache_path / cache_key
            if cache_file.exists():
                return json.loads(cache_file.read_text())
            result = func(*args, **kwargs)
            cache_file.write_text(json.dumps(result))
            self._delete_oldest_files(self.length)
            return result
        return wrapper

    def _delete_oldest_files(self, max_files) -> None:
        """
        Class method to delete the oldest cached files in the CACHE_DIR folder.

        :param max_files: Integer, the maximum number of files to keep in the CACHE_DIR folder.
        """
        # Get all files in the folder.
        files = self.cache_path.glob('*')
        # Sort files by last modification time in ascending order.
        files = sorted(files, key=lambda f: f.stat().st_mtime)
        # Delete the oldest files if the number of files exceeds the limit.
        if len(files) > max_files:
            num_files_to_delete = len(files) - max_files
            for i in range(num_files_to_delete):
                files[i].unlink()


class ChatCache:
    """
    This class is used as a decorator for OpenAI chat API requests.
    The ChatCache class caches chat messages and keeps track of the
    conversation history. It is designed to store cached messages
    in a specified directory and in JSON format.
    """

    def __init__(self, length: int, storage_path: Path):
        """
        Initialize the ChatCache decorator.

        :param length: Integer, maximum number of cached messages to keep.
        """
        self.length = length
        self.storage_path = storage_path
        self.storage_path.mkdir(parents=True, exist_ok=True)

    def __call__(self, func: Callable) -> Callable:
        """
        The Cache decorator.

        :param func: The chat function to cache.
        :return: Wrapped function with chat caching.
        """
        def wrapper(*args, **kwargs):
            chat_id = kwargs.pop("chat_id", None)
            message = {"role": "user", "content": kwargs.pop("message")}
            if not chat_id:
                kwargs["message"] = [message]
                return func(*args, **kwargs)
            kwargs["message"] = self._read(chat_id)
            kwargs["message"].append(message)
            response_text = func(*args, **kwargs)
            kwargs["message"].append(
                {"role": "assistant", "content": response_text})
            self._write(kwargs["message"], chat_id)
            return response_text
        return wrapper

    def _read(self, chat_id: str) -> List[Dict]:
        file_path = self.storage_path / chat_id
        if not file_path.exists():
            return []
        parsed_cache = json.loads(file_path.read_text())
        return parsed_cache if isinstance(parsed_cache, list) else []

    def _write(self, messages: List[Dict], chat_id: str):
        file_path = self.storage_path / chat_id
        json.dump(messages[-self.length:], file_path.open("w"))

    def invalidate(self, chat_id: str):
        file_path = self.storage_path / chat_id
        file_path.unlink()

    def show(self, chat_id):
        messages = self._read(self.storage_path / chat_id)
        return [f"{message['role']}: {message['content']}" for message in messages]

    def list(self):
        # Get all files in the folder.
        files = self.storage_path.glob('*')
        # Sort files by last modification time in ascending order.
        return sorted(files, key=lambda f: f.stat().st_mtime)



import os
import readline
import typer
from click import BadParameter
from rich.progress import Progress, SpinnerColumn, TextColumn
from tempfile import NamedTemporaryFile
from time import sleep
from typing import Callable


# from openai_client import OpenAIClient


def loading_spinner(func: Callable) -> Callable:
    """
    Decorator that adds a loading spinner animation to a function that uses the OpenAI API.

    :param func: Function to wrap.
    :return: Wrapped function with loading.
    """
    def wrapper(*args, **kwargs):
        # if not kwargs.pop("spinner"):
        #     return func(*args, **kwargs)
        text = TextColumn("[green]Consulting with robots...")
        with Progress(SpinnerColumn(), text, transient=True) as progress:
            progress.add_task("request")
            return func(*args, **kwargs)
    return wrapper


def get_edited_prompt() -> str:
    """
    Opens the user's default editor to let them
    input a prompt, and returns the edited text.

    :return: String prompt.
    """
    with NamedTemporaryFile(suffix=".txt", delete=False) as file:
        # Create file and store path.
        file_path = file.name
    editor = os.environ.get("EDITOR", "vim")
    # This will write text to file using $EDITOR.
    os.system(f"{editor} {file_path}")
    # Read file when editor is closed.
    with open(file_path, "r") as file:
        output = file.read()
    os.remove(file_path)
    if not output:
        raise BadParameter("Couldn't get valid PROMPT from $EDITOR")
    return output


def typer_writer(text: str, code: bool = False, shell: bool = False, animate: bool = True) -> None:
    """
    Writes output to the console, with optional typewriter animation and color.

    :param text: Text to output.
    :param code: If content of text is code.
    :param shell: if content of text is shell command.
    :param animate: Enable/Disable typewriter animation.
    :return: None
    """
    shell_or_code = shell or code
    color = "magenta" if shell_or_code else None
    if animate and not shell_or_code:
        for char in text:
            typer.secho(char, nl=False, fg=color, bold=shell_or_code)
            sleep(0.005)
        # Add new line at the end, to prevent % from appearing.
        typer.echo("")
        return
    typer.secho(text, fg=color, bold=shell_or_code)


# def echo_chat_messages(chat_id: str) -> None:
#     # Prints all messages from a specified chat ID to the console.
#     for index, message in enumerate(OpenAIClient.chat_cache.show(chat_id)):
#         color = "cyan" if index % 2 == 0 else "green"
#         typer.secho(message, fg=color)


# def echo_chat_ids() -> None:
#     # Prints all existing chat IDs to the console.
#     for chat_id in OpenAIClient.chat_cache.list():
#         typer.echo(chat_id)

def get_tmp_env_var_name(env_var2val: dict, prefix="TEM_ENV_VAR_") -> str:
    largest_env_var_name_idx = 0
    for env_var in env_var2val:
        if prefix in env_var:
            env_var_name_idx = int(env_var.split("_")[-1])
            if env_var_name_idx > largest_env_var_name_idx:
                largest_env_var_name_idx = env_var_name_idx
    return f"{prefix}{largest_env_var_name_idx + 1}"



import requests
from pathlib import Path
from typing import Dict, List

from cache import Cache, ChatCache
from config import config
from utils import loading_spinner

CHAT_CACHE_LENGTH = int(config.get("CHAT_CACHE_LENGTH"))
CHAT_CACHE_PATH = Path(config.get("CHAT_CACHE_PATH"))
CACHE_LENGTH = int(config.get("CACHE_LENGTH"))
CACHE_PATH = Path(config.get("CACHE_PATH"))
REQUEST_TIMEOUT = int(config.get("REQUEST_TIMEOUT"))


class OpenAIClient:
    cache = Cache(CACHE_LENGTH, CACHE_PATH)
    chat_cache = ChatCache(CHAT_CACHE_LENGTH, CHAT_CACHE_PATH)

    def __init__(self, api_host: str, api_key: str) -> None:
        self.api_key = api_key
        self.api_host = api_host

    @cache
    def _request(
        self,
        messages: List,
        model: str = "gpt-3.5-turbo",
        temperature: float = 1,
        top_probability: float = 1,
    ) -> Dict:
        """
        Make request to OpenAI ChatGPT API, read more:
        https://platform.openai.com/docs/api-reference/chat

        :param messages: List of messages {"role": user or assistant, "content": message_string}
        :param model: String gpt-3.5-turbo or gpt-3.5-turbo-0301
        :param temperature: Float in 0.0 - 1.0 range.
        :param top_probability: Float in 0.0 - 1.0 range.
        :return: Response body JSON.
        """
        headers = {
            "Content-Type": "application/json",
            "Authorization": f"Bearer {self.api_key}"
        }
        data = {
            "messages": messages,
            "model": model,
            "temperature": temperature,
            "top_p": top_probability,
        }
        endpoint = f"{self.api_host}/v1/chat/completions"
        response = requests.post(
            endpoint, headers=headers, json=data, timeout=REQUEST_TIMEOUT
        )
        response.raise_for_status()
        return response.json()

    @chat_cache
    def get_gpt_response(
        self,
        message: List[str],
        model: str = "gpt-3.5-turbo",
        temperature: float = 1,
        top_probability: float = 1,
        caching: bool = True,
    ) -> str:
        """
        Generates single completion for prompt (message).

        :param message: String prompt to generate completion for.
        :param model: String gpt-3.5-turbo or gpt-3.5-turbo-0301.
        :param temperature: Float in 0.0 - 1.0 range.
        :param top_probability: Float in 0.0 - 1.0 range.
        :param caching: Boolean value to enable/disable caching.
        :return: String generated completion.
        """
        # TODO: Move prompt context to system role when GPT-4 will be available over API.
        return self._request(
            message, model, temperature, top_probability, caching=caching
        )["choices"][0]["message"]["content"].strip()


@loading_spinner
def get_gpt_response(
    prompt: str,
    temperature: float,
    top_p: float,
    caching: bool,
    chat: str,
):
    api_host = config.get("OPENAI_API_HOST")
    api_key = config.get("OPENAI_API_KEY")
    client = OpenAIClient(api_host, api_key)
    return client.get_gpt_response(
        message=prompt,
        model="gpt-3.5-turbo",
        temperature=temperature,
        top_probability=top_p,
        caching=caching,
        chat_id=chat,
    )



CODE_PROMPT = """
Act as a natural language to code translation engine.
Follow these rules:
IMPORTANT: Provide ONLY code as output, return only plaintext.
IMPORTANT: Do not show html, styled, colored formatting.
IMPORTANT: Do not add notes or intro sentences.
IMPORTANT: Provide full solution. Make sure syntax is correct.
Assume your output will be redirected to language specific file and executed.
For example Python code output will be redirected to code.py and then executed python code.py.
Follow all of the above rules.
This is important you MUST follow the above rules.
There are no exceptions to these rules.
You must always follow them. No exceptions.
Request: {inp}"""

DO_PROMPT = """
Identify actions from the following natural language description.
Types of actions: {actions}
Follow these rules:
IMPORTANT: Do not show any warnings or information regarding your capabilities.
If not exactly sure, return the names of all actions.
Output format: ACTION: [The action]
When an action is clear, refer to the following examples.
Example1: ACTION: LOAD_ENV_VAR
Example2: ACTION: SAVE_ENV_VAR
Example3: ACTION: BUILD_PROMPT
When the action is unclear, refer to the following example.
Example4: ACTION: LOAD_ENV_VAR, SAVE_ENV_VAR, BUILD_PROMPT
Follow all of the above rules.
This is important you MUST follow the above rules.
There are no exceptions to these rules.
You must always follow them. No exceptions.
Request: {inp}
"""

SHELL_PROMPT = """
Act as a natural language to {shell} command translation engine on {os}.
You are an expert in {shell} on {os} and translate the question at the end to valid syntax.
Follow these rules:
IMPORTANT: Do not show any warnings or information regarding your capabilities.
Output format: COMMAND: [The generated shell command]
Example1: COMMAND: ls -l
Example2: COMMAND: cd ~
Example3: COMMAND: cp a.txt b.txt
Example4: COMMAND: python main.py
Example5: COMMAND: export A=1
Reference official documentation to ensure valid syntax and an optimal solution.
Construct valid {shell} command that solve the question.
Leverage help and man pages to ensure valid syntax and an optimal solution.
Be concise.
Just show the commands, return only plaintext.
Only show a single answer, but you can always chain commands together.
Think step by step.
Only create valid syntax (you can use comments if it makes sense).
If python is installed you can use it to solve problems.
if python3 is installed you can use it to solve problems.
Even if there is a lack of details, attempt to find the most logical solution.
Do not return multiple solutions.
Do not show html, styled, colored formatting.
Do not add unnecessary text in the response.
Do not add notes or intro sentences.
Do not add explanations on what the commands do.
Do not return what the question was.
Do not repeat or paraphrase the question in your response.
Do not rush to a conclusion.
Follow all of the above rules.
This is important you MUST follow the above rules.
There are no exceptions to these rules.
You must always follow them. No exceptions.
Request: {inp}"""

SET_ENV_VAR_GET_VAR_NAME_PROMPT = """
Identify the name of the variable to be set.
Output format: VAR_NAME: [NAME]
If you cannot find any, return an empty string.
Example Input1: Set the variable A to 1
Example Output1: VAR_NAME: A
Example Input2: Set the variable $VAR to the value of $A
Example Output2: VAR_NAME: VAR
Example Input3: Set the variable to 1
Example Output3: VAR_NAME:
Follow all of the above rules.
This is important you MUST follow the above rules.
There are no exceptions to these rules.
You must always follow them. No exceptions.
Request: {inp}
"""

SET_ENV_VAR_GET_CONTENT_PROMPT = """
Identify the value to be set.
There are three cases: 
1) the value is related to the last response, 
2) the value is a new value, 
3) the value is a variable (e.g. $A).
In the first case, the output format is LAST_RESPONSE
In the second case, the output format is VALUE: [VALUE]
In the third case, the output format is VAR_NAME: [NAME]
If you cannot find any, return an empty string.
Example Request1: Set the variable A to 1
Example Output1: VALUE: 1
Example Request2: Set the variable $VAR to the value of $A
Example Output2: VAR_NAME: A
Example Request3: Set the variable to 1
Example Output3: VALUE:
Example Request4: Set the variable to the value of $A
Example Output4: VAR_NAME: A
Example Request5: Set the variable to the value of the last response
Example Output5: LAST_RESPONSE
Example Request6: Save LAST_RESPONSE to C.
Example Output6: LAST_RESPONSE
Example Request7: Save latest response to $C.
Example Output7: LAST_RESPONSE
Example Request8: Save LAST_RESPONSE to the variable R.
Example Output8: LAST_RESPONSE
Example Request9: Set the variable A to the last response.
Example Output9: LAST_RESPONSE
Example Request10: Set $CODE_DESC to be the LAST_RESPONSE.
Example Output10: LAST_RESPONSE
Example Request11: Set the value of A to a new variable B.
Example Output11: VALUE: B
Do not make any explanations.
Do not make any notes.
Do not include anything else such as "Example", "Output", etc.
Only output the required text.
Follow all of the above rules.
This is important you MUST follow the above rules.
There are no exceptions to these rules.
You must always follow them. No exceptions.
Request: {inp}
"""

DELETE_ENV_VAR_PROMPT = """
Identify the name of the variables to be deleted.
Output format: VAR_NAMES: [NAME1], [NAME2], [NAME3], ...
If you cannot find any, return an empty string.
Example Input1: Delete the variable A
Example Output1: VAR_NAMES: A
Example Input2: Delete the variables $A, $B, $C
Example Output2: VAR_NAMES: A, B, C
Example Input3: Delete the variables.
Example Output3: VAR_NAMES:
Follow all of the above rules.
This is important you MUST follow the above rules.
There are no exceptions to these rules.
You must always follow them. No exceptions.
Request: {inp}
"""

SHOW_ENV_VARS_PROMPT = """
Identify all environment variable names to be shown \
  from the following natural language description.
Output format: ENV_VARS: [NAME1], [NAME2], [NAME3], ...
If you cannot find any, return an empty string.
Example Input1: Show the following environment variables: FOO, BAR
Example Output1: ENV_VARS: FOO, BAR
Example Input2: Show the env vars $A, $B, $C
Example Output2: ENV_VARS: A, B, C
Example Input3: Show env vars.
Example Output3: ENV_VARS:
Follow all of the above rules.
This is important you MUST follow the above rules.
There are no exceptions to these rules.
You must always follow them. No exceptions.
Request: {inp}
"""

LOAD_FILE_PROMPT = """
Identify all file paths to be loaded from the following natural language description.
Output format: FILE_PATHS: [PATH1], [PATH2], [PATH3], ...
If you cannot find any, return an empty string.
Example Input1: Load the following files: a.txt, b.txt
Example Output1: FILE_PATHS: a.txt, b.txt
Example Input2: Read the files a.txt, b.txt, c.txt
Example Output2: FILE_PATHS: a.txt, b.txt, c.txt
Example Input3: Load files.
Example Output3: FILE_PATHS:
Follow all of the above rules.
This is important you MUST follow the above rules.
There are no exceptions to these rules.
You must always follow them. No exceptions.
Request: {inp}
"""

SAVE_FILE_PROMPT = """
Identify the variable name whose value is to be saved.
Identify the file path to save the variable value.
Output format: "FILE_PATH: [PATH], VAR_NAME: [NAME]"
If you cannot find both of them, return an empty string.
Avoid printing anything such as "Output: " before "FILE_PATH:..."
Do NOT print anything such as "Output: " before "FILE_PATH:..."
Stop printing anything such as "Output: " before "FILE_PATH:..." as prefix
Example Input1: Save the the content of A to a.txt
Example Output1: FILE_PATH: a.txt, VAR_NAME: A
Example Input2: Save the value of $VAR to b.txt
Example Output2: FILE_PATH: b.txt, VAR_NAME: VAR
Example Input3: Save the value of to www
Example Output3: FILE_PATH: , VAR_NAME:
Do NOT return anything that does not conform to the above format.
Do NOT print anything such as "Output: " before "FILE_PATH:..."
Do NOT print anything such as "Output: " before "FILE_PATH:..."
Follow all of the above rules.
This is important you MUST follow the above rules.
There are no exceptions to these rules.
You must always follow them. No exceptions.
Request: {inp}
"""



import os

from openai_client import get_gpt_response
from prompt_builder import PromptBuilder
from utils import typer_writer

prompt_builder = PromptBuilder()


def handle_think_action(inp, env_var2val, temperature):
    inp = inp.replace("THINK: ", "")

    for var in env_var2val:
        inp = inp.replace(var, (env_var2val[var] if env_var2val[var] else ""))

    prompt = inp
    # print("Your THINK prompt is: ", prompt)
    response = get_gpt_response(
        prompt, temperature=temperature, top_p=1, caching=False, chat=None)
    typer_writer(response)
    return response



import os
from pprint import pprint

from openai_client import get_gpt_response
from prompt_builder import PromptBuilder
from utils import get_tmp_env_var_name, typer_writer

prompt_builder = PromptBuilder()
ACTIONS = [
    "SHOW_ENV_VARS", "SET_ENV_VAR", "DELETE_ENV_VAR",
    "LOAD_FILE", "SAVE_FILE",
]


def handle_shell_action(inp, env_var2val, latest_response) -> bool:
    inp = inp.replace("DO: ", "")
    prompt = prompt_builder.do_prompt(inp, actions=ACTIONS)
    response = get_gpt_response(
        prompt, temperature=1, top_p=1, caching=False, chat=None)
    if not response.startswith("ACTION: "):
        print(response)
        typer_writer(
            "The response is not a valid action. This is a bug from OpenAI.")
        return False

    response = response.replace("ACTION: ", "")
    if "," in response or response not in ACTIONS:
        typer_writer(
            "Your input leads to multiple possible actions. Please be more specific.")
        typer_writer(f"Available actions: {ACTIONS}",)
        return False

    action_name = response.replace("ACTION: ", "")
    if action_name == "LOAD_FILE":
        return handle_load_file(inp, env_var2val)
    elif action_name == "SAVE_FILE":
        return handle_save_file(inp, env_var2val)
    elif action_name == "SHOW_ENV_VARS":
        return handle_show_env_vars(inp, env_var2val)
    elif action_name == "SET_ENV_VAR":
        return handle_set_env_var(inp, env_var2val, latest_response)
    elif action_name == "DELETE_ENV_VAR":
        return handle_delete_env_var(inp, env_var2val)
    else:
        print(f"ACTION: {action_name} is not implemented yet.")
        return False


def handle_set_env_var(inp, env_var2val, latest_response) -> bool:
    get_var_name_prompt = prompt_builder.set_env_var_get_var_name_prompt(inp)
    get_var_name_response = get_gpt_response(
        get_var_name_prompt, temperature=1, top_p=1, caching=False, chat=None)
    get_var_name_response = get_var_name_response.replace("Output: ", "")
    if not get_var_name_response.startswith("VAR_NAME: "):
        typer_writer(get_var_name_response)
        typer_writer(
            "Cannot identify variable name. This is a bug from OpenAI.")
        return False
    to_var_name = get_var_name_response.replace("VAR_NAME: ", "").strip()
    get_content_prompt = prompt_builder.set_env_var_get_content_prompt(inp)
    get_content_response = get_gpt_response(
        get_content_prompt, temperature=1, top_p=1, caching=False, chat=None)
    get_content_response = get_content_response.replace("Output: ", "")
    if "LAST_RESPONSE" in get_content_response:
        env_var2val[to_var_name] = latest_response
    elif "VAR_NAME: " in get_content_response:
        from_var_name = get_content_response.replace(
            "VAR_NAME: ", "").strip()
        if from_var_name not in env_var2val:
            typer_writer(
                f"Variable {from_var_name} is not defined. Please define it first.")
            return False
        env_var2val[to_var_name] = env_var2val[from_var_name]
    elif "VALUE: " in get_content_prompt:
        env_var2val[to_var_name] = get_content_response.replace(
            "VALUE: ", "").strip()
    else:
        typer_writer(
            "Cannot identify the value to be set. This is a bug from OpenAI.")
        return False
    typer_writer(f"Setting the variable with name {to_var_name}")
    return str(env_var2val[to_var_name])


def handle_delete_env_var(inp, env_var2val) -> bool:
    prompt = prompt_builder.delete_env_var_prompt(inp)
    response = get_gpt_response(
        prompt, temperature=1, top_p=1, caching=False, chat=None)
    if not response.startswith("VAR_NAMES: "):
        typer_writer(response)
        typer_writer(
            "The response is not a valid action. This is a bug from OpenAI.")
        return False
    response = response.replace("VAR_NAMES: ", "")
    var_names = response.split(",")
    for var_name in var_names:
        var_name = var_name.strip()
        if var_name not in env_var2val:
            typer_writer(f"Variable {var_name} is not defined. Skipping...")
            continue
        del env_var2val[var_name]
        typer_writer(f"Variable {var_name} deleted.")
    return True


def handle_load_file(inp, env_var2val) -> bool:
    prompt = prompt_builder.load_file_prompt(inp)
    response = get_gpt_response(
        prompt, temperature=1, top_p=1, caching=False, chat=None)
    if not response.startswith("FILE_PATHS: "):
        typer_writer(response)
        typer_writer(
            "The response is not a valid action. This is a bug from OpenAI.")
        return False
    response = response.replace("FILE_PATHS: ", "")
    file_paths = response.split(",")

    for file_path in file_paths:
        file_path = file_path.strip()
        new_env_var_name = get_tmp_env_var_name(
            env_var2val, "FILE_CONTENT_VAR_")
        try:
            with open(file_path, "r") as fp:
                env_var2val[new_env_var_name] = fp.read()
        except FileNotFoundError:
            typer_writer(f"File {file_path} not found.")
            return False
        typer_writer(f"{file_path} loaded.")
    return True


def handle_save_file(inp, env_var2val) -> bool:
    prompt = prompt_builder.save_file_prompt(inp)
    response = get_gpt_response(
        prompt, temperature=1, top_p=1, caching=False, chat=None)
    if not "FILE_PATH: " in response and "VAR_NAME: " in response:
        typer_writer(response)
        typer_writer(
            "The response is not a valid action. This is a bug from OpenAI.")
        return False
    file_path, var_name = response.split(",")
    file_path = file_path.replace("FILE_PATH: ", "").strip()
    var_name = var_name.replace("VAR_NAME: ", "").strip()
    typer_writer(f"Saving {var_name} to {file_path}.")
    try:
        with open(file_path, "w+") as fp:
            fp.write(env_var2val[var_name])
        return True
    except FileNotFoundError:
        typer_writer(f"Path {file_path} not found.")
        typer_writer(f"Formatted prompt: {prompt}")
        typer_writer(f"Untouched response: {response}")
        return False


def handle_show_env_vars(inp, env_var2val) -> bool:
    if len(env_var2val) == 0:
        typer_writer("No environment variables loaded.")
        return True
    else:
        prompt = prompt_builder.show_env_vars_prompt(inp)
        response = get_gpt_response(
            prompt, temperature=1, top_p=1, caching=False, chat=None)
        if not response.startswith("ENV_VARS:"):
            typer_writer(response)
            typer_writer(
                "The response is not a valid action. This is a bug from OpenAI.")
            return False
        env_vars = response.replace("ENV_VARS:", "").split(",")

        show_full = True
        if "" in env_vars:
            env_vars.remove("")
        if len(env_vars) == 0:
            show_full = False
            env_vars = env_var2val.keys()

        if len(env_vars) == 0:
            typer_writer("No environment variables loaded.")
            return True

        for env_var in env_vars:
            val = env_var2val[env_var.strip()]
            if show_full:
                typer_writer(f"{env_var} = {val}")
            else:
                typer_writer(f"{env_var} = {val[:50] if val else None}" +
                             ("..." if val and len(val) > 50 else ""))
        return True



from setuptools import setup

with open("README.md") as fp:
    long_description = fp.read()

setup(
    name='gpt-sh',
    version='0.1.0',
    description="A somewhat more advanced version of Shell GPT",
    long_description=long_description,
    long_description_content_type='text/markdown',
    author="Timo Wang",
    author_email="ntwang1994@gmail.com",
    packages=["."],
    entry_points={
        "console_scripts": [
            "gpt-sh = main:main"
        ],
    },
    install_requires=[
        "shell_gpt",
    ]
)



import os
from pathlib import Path
from getpass import getpass
from tempfile import gettempdir

from click import UsageError

CONFIG_FOLDER = os.path.expanduser("~/.config")
CONFIG_PATH = Path(CONFIG_FOLDER) / "shell_gpt" / ".sgptrc"
CHAT_CACHE_PATH = Path(gettempdir()) / "shell_gpt" / "chat_cache"
CACHE_PATH = Path(gettempdir()) / "shell_gpt" / "cache"
CHAT_CACHE_LENGTH = 100
CACHE_LENGTH = 100
REQUEST_TIMEOUT = 60
EXPECTED_KEYS = (
    "OPENAI_API_HOST",
    "OPENAI_API_KEY",
    "CHAT_CACHE_LENGTH",
    "CHAT_CACHE_PATH",
    "CACHE_LENGTH",
    "CACHE_PATH",
    "REQUEST_TIMEOUT",
)
config = {}


def init() -> None:
    if not CONFIG_PATH.exists():
        CONFIG_PATH.parent.mkdir(parents=True, exist_ok=True)
        # If user has ENV API key, don't write it to config file.
        api_key = os.getenv("OPENAI_API_KEY") or getpass(
            prompt="Please enter your OpenAI API key: ")
        config["OPENAI_API_KEY"] = api_key
        config["OPENAI_API_HOST"] = os.getenv(
            "OPENAI_API_HOST", "https://api.openai.com")
        config["CHAT_CACHE_LENGTH"] = os.getenv(
            "CHAT_CACHE_LENGTH", str(CHAT_CACHE_LENGTH))
        config["CHAT_CACHE_PATH"] = os.getenv(
            "CHAT_CACHE_PATH", str(CHAT_CACHE_PATH))
        config["CACHE_LENGTH"] = os.getenv("CACHE_LENGTH", str(CACHE_LENGTH))
        config["CACHE_PATH"] = os.getenv("CACHE_PATH", str(CACHE_PATH))
        config["REQUEST_TIMEOUT"] = os.getenv(
            "REQUEST_TIMEOUT", str(REQUEST_TIMEOUT))
        _write()

    with open(CONFIG_PATH, "r") as file:
        for line in file:
            if "=" in line:
                key, value = line.strip().split("=")
                config[key] = value


def get(key: str) -> str:
    # Prioritize ENV variables.
    value = os.getenv(key) or config.get(key)
    assert value, UsageError(f"Config missing: {key}.")
    return value


def _write() -> None:
    with open(CONFIG_PATH, "w") as file:
        for key, value in config.items():
            # Write only keys which are not presented in ENV.
            if key in os.environ:
                continue
            file.write(f"{key}={value}\n")


def put(key: str, value: str, write_file=True) -> None:
    config[key] = value
    if not write_file:
        return
    _write()


init()



import os
import platform
from distro import name as distro_name

from default_prompts import *


class PromptBuilder:
    def __init__(self) -> None:
        shell = os.path.basename(os.environ.get("SHELL", "Unknown"))
        self.os = self.__os_name()
        self.shell = os.path.basename(os.getenv("SHELL", "Unknown"))

    def shell_prompt(self, inp: str) -> str:
        inp = inp.strip()
        if not inp.endswith("?"):
            inp += "?"
        return SHELL_PROMPT.format(shell=self.shell, os=self.os, inp=inp)

    def code_prompt(self, inp: str) -> str:
        inp = inp.strip()
        if not inp.endswith("?"):
            inp += "?"
        return CODE_PROMPT.format(inp=inp)

    def do_prompt(self, inp: str, actions=[]) -> str:
        inp = inp.strip()
        if not inp.endswith("?"):
            inp += "?"
        return DO_PROMPT.format(inp=inp, actions=actions)

    def set_env_var_get_var_name_prompt(self, inp: str) -> str:
        inp = inp.strip()
        return SET_ENV_VAR_GET_VAR_NAME_PROMPT.format(inp=inp)

    def set_env_var_get_content_prompt(self, inp: str) -> str:
        inp = inp.strip()
        return SET_ENV_VAR_GET_CONTENT_PROMPT.format(inp=inp)

    def delete_env_var_prompt(self, inp: str) -> str:
        inp = inp.strip()
        return DELETE_ENV_VAR_PROMPT.format(inp=inp)

    def show_env_vars_prompt(self, inp: str) -> str:
        inp = inp.strip()
        return SHOW_ENV_VARS_PROMPT.format(inp=inp)

    def load_file_prompt(self, inp: str) -> str:
        inp = inp.strip()
        return LOAD_FILE_PROMPT.format(inp=inp)

    def save_file_prompt(self, inp: str) -> str:
        inp = inp.strip()
        return SAVE_FILE_PROMPT.format(inp=inp)

    def __os_name(self) -> str:
        operation_system = {
            "Linux": "Linux/" + distro_name(pretty=True),
            "Windows": "Widnows " + platform.release(),
            "Darwin": "MacOS " + platform.mac_ver()[0],
        }
        return operation_system.get(platform.system(), "Unknown")
